package dev.shadowsoffire.apotheosis.adventure.socket.gem;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function8;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.compat.GameStagesCompat;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityClamp;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketHelper;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemInstance;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import javax.annotation.Nullable;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.CommonComponents;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.network.chat.Style;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.ItemStack;
import org.apache.commons.lang3.tuple.Pair;

public class Gem implements CodecProvider<Gem>, WeightedDynamicRegistry.ILuckyWeighted, WeightedDynamicRegistry.IDimensional, RarityClamp, GameStagesCompat.IStaged {
  public static final Codec<Gem> CODEC;
  
  protected final int weight;
  
  protected final float quality;
  
  protected final Set<ResourceLocation> dimensions;
  
  protected final List<GemBonus> bonuses;
  
  protected final boolean unique;
  
  @Nullable
  protected final Set<String> stages;
  
  protected final transient Map<LootCategory, GemBonus> bonusMap;
  
  protected final transient int uuidsNeeded;
  
  protected final transient LootRarity minRarity;
  
  protected final transient LootRarity maxRarity;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group((App)Codec.intRange(0, 2147483647).fieldOf("weight").forGetter(WeightedDynamicRegistry.ILuckyWeighted::getWeight), (App)PlaceboCodecs.nullableField(Codec.floatRange(0.0F, Float.MAX_VALUE), "quality", Float.valueOf(0.0F)).forGetter(WeightedDynamicRegistry.ILuckyWeighted::getQuality), (App)PlaceboCodecs.nullableField(PlaceboCodecs.setOf(ResourceLocation.f_135803_), "dimensions", Collections.emptySet()).forGetter(WeightedDynamicRegistry.IDimensional::getDimensions), (App)PlaceboCodecs.nullableField(LootRarity.CODEC, "min_rarity").forGetter(()), (App)PlaceboCodecs.nullableField(LootRarity.CODEC, "max_rarity").forGetter(()), (App)GemBonus.CODEC.listOf().fieldOf("bonuses").forGetter(Gem::getBonuses), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "unique", Boolean.valueOf(false)).forGetter(Gem::isUnique), (App)PlaceboCodecs.nullableField(PlaceboCodecs.setOf((Codec)Codec.STRING), "stages").forGetter(())).apply((Applicative)inst, Gem::new));
  }
  
  public Gem(int weight, float quality, Set<ResourceLocation> dimensions, Optional<LootRarity> minRarity, Optional<LootRarity> maxRarity, List<GemBonus> bonuses, boolean unique, Optional<Set<String>> stages) {
    this.weight = weight;
    this.quality = quality;
    this.dimensions = dimensions;
    this.bonuses = bonuses;
    this.unique = unique;
    this.stages = stages.orElse(null);
    Preconditions.checkArgument(!bonuses.isEmpty(), "No bonuses were provided.");
    this


      
      .bonusMap = (Map<LootCategory, GemBonus>)bonuses.stream().mapMulti((gemData, mapper) -> {
          for (LootCategory c : gemData.getGemClass().types())
            mapper.accept(Pair.of(c, gemData)); 
        }).collect(Collectors.toMap(Pair::getLeft, Pair::getRight));
    this.uuidsNeeded = this.bonuses.stream().mapToInt(GemBonus::getNumberOfUUIDs).max().orElse(0);
    if (minRarity.isPresent()) {
      this.minRarity = minRarity.get();
    } else {
      Objects.requireNonNull(bonuses.get(0));
      this.minRarity = RarityRegistry.INSTANCE.getValues().stream().filter((GemBonus)bonuses.get(0)::supports).min(LootRarity::compareTo).get();
    } 
    if (maxRarity.isPresent()) {
      this.maxRarity = maxRarity.get();
    } else {
      Objects.requireNonNull(bonuses.get(0));
      this.maxRarity = RarityRegistry.INSTANCE.getValues().stream().filter((GemBonus)bonuses.get(0)::supports).max(LootRarity::compareTo).get();
    } 
    Preconditions.checkArgument((this.minRarity.ordinal() <= this.maxRarity.ordinal()), "The min rarity must be <= the max rarity.");
  }
  
  public int getNumberOfUUIDs() {
    return this.uuidsNeeded;
  }
  
  public void addInformation(ItemStack gem, LootRarity rarity, Consumer<Component> list) {
    if (isUnique()) {
      list.accept(Component.m_237115_("text.apotheosis.unique").m_130948_(Style.f_131099_.m_178520_(13056274)));
      list.accept(CommonComponents.f_237098_);
    } 
    Style style = Style.f_131099_.m_178520_(720650);
    list.accept(Component.m_237115_("text.apotheosis.socketable_into").m_130948_(style));
    addTypeInfo(list, this.bonusMap.keySet().toArray());
    list.accept(CommonComponents.f_237098_);
    list.accept(Component.m_237115_("text.apotheosis.when_socketed_in").m_130940_(ChatFormatting.GOLD));
    for (GemBonus bonus : this.bonuses) {
      if (!bonus.supports(rarity))
        continue; 
      Component modifComp = bonus.getSocketBonusTooltip(gem, rarity);
      MutableComponent mutableComponent = Component.m_237110_("text.apotheosis.dot_prefix", new Object[] { Component.m_237110_("%s: %s", new Object[] { Component.m_237115_("gem_class." + bonus.getGemClass().key()), modifComp }) }).m_130940_(ChatFormatting.GOLD);
      list.accept(mutableComponent);
    } 
  }
  
  public boolean canApplyTo(ItemStack socketed, ItemStack gem, LootRarity rarity) {
    if (isUnique()) {
      List<Gem> gems = SocketHelper.getGems(socketed).streamValidGems().map(GemInstance::gem).map(DynamicHolder::get).toList();
      if (gems.contains(this))
        return false; 
    } 
    return isValidIn(socketed, gem, rarity);
  }
  
  public boolean isValidIn(ItemStack socketed, ItemStack gem, LootRarity rarity) {
    LootCategory cat = LootCategory.forItem(socketed);
    return (!cat.isNone() && this.bonusMap.containsKey(cat) && ((GemBonus)this.bonusMap.get(cat)).supports(rarity));
  }
  
  public Optional<GemBonus> getBonus(LootCategory cat, LootRarity rarity) {
    return Optional.<GemBonus>ofNullable(this.bonusMap.get(cat)).filter(b -> b.supports(rarity));
  }
  
  public String toString() {
    return String.format("Gem: %s", new Object[] { getId() });
  }
  
  public static String fmt(float f) {
    return Affix.fmt(f);
  }
  
  public float getQuality() {
    return this.quality;
  }
  
  public int getWeight() {
    return this.weight;
  }
  
  public Set<ResourceLocation> getDimensions() {
    return this.dimensions;
  }
  
  public LootRarity getMaxRarity() {
    return this.maxRarity;
  }
  
  public LootRarity getMinRarity() {
    return this.minRarity;
  }
  
  public List<GemBonus> getBonuses() {
    return this.bonuses;
  }
  
  public boolean isUnique() {
    return this.unique;
  }
  
  public Gem validate(ResourceLocation key) {
    Preconditions.checkNotNull(this.dimensions);
    Preconditions.checkArgument((this.maxRarity.ordinal() >= this.minRarity.ordinal()));
    RarityRegistry.INSTANCE.getValues().stream().filter(r -> (r.isAtLeast(this.minRarity) && r.isAtMost(this.maxRarity))).forEach(r -> Preconditions.checkArgument(this.bonuses.stream().allMatch(())));
    return this;
  }
  
  public Set<String> getStages() {
    return this.stages;
  }
  
  public Codec<? extends Gem> getCodec() {
    return CODEC;
  }
  
  public static void addTypeInfo(Consumer<Component> list, Object... types) {
    Arrays.sort(types, (c1, c2) -> ((LootCategory)c1).getName().compareTo(((LootCategory)c2).getName()));
    Style style = Style.f_131099_.m_178520_(720650);
    if (types.length != LootCategory.BY_ID.size() - 1) {
      StringBuilder sb = new StringBuilder();
      int i = 0;
      while (i < types.length) {
        int rem = Math.min(3, types.length - i);
        Object[] args = new Object[rem];
        for (int r = 0; r < rem; r++) {
          sb.append("%s, ");
          args[r] = Component.m_237115_(((LootCategory)types[i + r]).getDescIdPlural());
        } 
        list.accept(Component.m_237110_("text.apotheosis.dot_prefix", new Object[] { Component.m_237110_(sb.substring(0, sb.length() - 2), args) }).m_130948_(style));
        sb.setLength(0);
        i += rem;
      } 
    } else {
      list.accept(Component.m_237110_("text.apotheosis.dot_prefix", new Object[] { Component.m_237115_("text.apotheosis.anything") }).m_130948_(style));
    } 
  }
  
  public final ResourceLocation getId() {
    return GemRegistry.INSTANCE.getKey(this);
  }
}
